/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            mov_encoder_writer.cc
 *
 *  Sun May 22 12:51:36 CEST 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include <config.h>
#include "mov_encoder_writer.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <pthread.h>
#include <semaphore.h>

#include <errno.h>

#include <string>
using namespace std;

#include "miav_config.h"

#include <time.h>

#include "multiplexer.h"

MovEncoderWriter::MovEncoderWriter(const char* cpr, 
                                   FramePriorityQueue *v_q, pthread_mutex_t *v_m, sem_t *v_s, 
                                   FramePriorityQueue *a_q, pthread_mutex_t *a_m, sem_t *a_s, 
                                   Info *i)
{
  info = i;
  info->info("MovEncoderWriter");

  // Create path and filename
  char fname[256];
  string *server_root;
  char birthmonth[3];
  char date[32];

  // Get server root
  server_root = config->readString("server_movie_root");

  // Copy the bytes representing the birth month from the cpr
  // [dd][mm][yy]-[nn][nn]
  strncpy(birthmonth, &cpr[2], 2);
  birthmonth[2] = 0;

  // Create date (today) in [yyyy][mm][dd]
  struct tm *ltime;
  time_t t = time(NULL);
  ltime = localtime(&t);
  sprintf(date, "%.4d%.2d%.2d", 
          ltime->tm_year + 1900, 
          ltime->tm_mon, 
          ltime->tm_mday);

  sprintf(fname, "%s/%s/%s/%s-%s-", server_root->c_str(), birthmonth, cpr, cpr, date);

  file = new File(fname, "mpg", info);

  video_queue = v_q;
  video_sem = v_s;
  video_mutex = v_m;

  audio_queue = a_q;
  audio_sem = a_s;
  audio_mutex = a_m;

  video_frame_number = 0;
  audio_frame_number = 0;

  running = true;
}

MovEncoderWriter::~MovEncoderWriter()
{
  info->info("~MovEncoderWriter");
  delete file;
}


void MovEncoderWriter::thread_main()
{
  info->info("MovEncoderWriter::run");

  Multiplexer multiplexer(file, info, &running, 
                          video_queue, video_mutex, video_sem,
                          audio_queue, audio_mutex, audio_sem);
  multiplexer.multiplex();

  info->info("MovEncoderWriter::stop");
}

